local LocalDatabase, _, SavedVars = unpack(select(2, ...))

--<<IMPORTANT STUFF>>-------------------------------------------------------------------------------<<>>

    local Modified_Plates       = {}
    local Visible_Plates        = {}
    local DefaultHeight         = 32
    local DefaultWidth          = 16

    local WorldFrame            = WorldFrame
    local Tooltip               = CreateFrame("GameTooltip", "KibQuestTooltip", nil, "GameTooltipTemplate")

--<<SET QUEST ICON>>--------------------------------------------------------------------------------<<>>

    local function SetQuestIcon(plateFrame)     
        plateFrame.Quest_Icon = plateFrame:CreateTexture(nil, "OVERLAY")                                   
        plateFrame.Quest_Icon:SetTexture("Interface\\AddOns\\Kib_QuestMobs\\media\\questicon_" .. SavedVars.TextureVariant)
        plateFrame.Quest_Icon:SetSize(DefaultWidth * SavedVars.ScaleOffset, DefaultHeight * SavedVars.ScaleOffset)
        plateFrame.Quest_Icon:SetPoint("BOTTOM", plateFrame, "TOP")
        plateFrame.Quest_Icon:Hide()

        Modified_Plates[#Modified_Plates + 1] = plateFrame
    end

--<<SET QUEST ICON COLOUR>>--------------------------------------------------------------------------------<<>>

    local function SetQuestIconColour(plateFrame)
        if plateFrame.QuestType == 1 then
            local Colour = SavedVars.AreaQuestcolor
            plateFrame.Quest_Icon:SetVertexColor(Colour.r, Colour.g, Colour.b, SavedVars.Alpha)
        elseif plateFrame.QuestType == 2 then
            local Colour = SavedVars.NormalQuestcolor
            plateFrame.Quest_Icon:SetVertexColor(Colour.r, Colour.g, Colour.b, SavedVars.Alpha)
        end
    end

--<<SCAN A PLATE FOR QUESTS>>-----------------------------------------------------------------------<<>>
    
    local function ScanForQuests(plateID)      
        local plateStats = Visible_Plates[plateID]
        if not plateStats then return end

        local plateFrame = plateStats.unitFrame
        plateFrame.QuestType = 0

        local QuestNoObjectives = nil

        Tooltip:SetOwner(WorldFrame, "ANCHOR_NONE")
        Tooltip:SetHyperlink("unit:" .. plateStats.unitGUID)
    
        for i = 3, Tooltip:NumLines() do
            local line = _G["KibQuestTooltipTextLeft" .. i]
            local text = line:GetText()
            local text_r, text_g, text_b = line:GetTextColor()

            if text_b == 0 and text_r > 0.99 and text_g > 0.82 then 
                QuestNoObjectives = true
            else
                local Unit_Name, Progress = string.match(text, "^ ([^ ]-) ?%- (.+)$")

                if Unit_Name and (Unit_Name == "" or Unit_Name == LocalDatabase.Player_Name) then
                    if Progress then
                        local current, goal = string.match(Progress, "(%d+)/(%d+)")

                        if current and (goal and current ~= goal) then plateFrame.QuestType = 2 end
                        QuestNoObjectives = nil 
                    end
                end
            end
        end

        if QuestNoObjectives then plateFrame.QuestType = 1 end 

        if plateFrame.QuestType > 0 then
            if not plateFrame.Quest_Icon then SetQuestIcon(plateFrame) end 
            plateFrame.Quest_Icon:Show()
            SetQuestIconColour(plateFrame)
        else
            if plateFrame.Quest_Icon then plateFrame.Quest_Icon:Hide() end
        end
    end

--<<HANDLE EVENTS>>---------------------------------------------------------------------------------<<>>
        
    function LocalDatabase.QUEST_LOG_UPDATE() for plateID in pairs(Visible_Plates) do ScanForQuests(plateID) end end

    function LocalDatabase.QUEST_TURNED_IN() for plateID in pairs(Visible_Plates) do ScanForQuests(plateID) end end
    
    function LocalDatabase.NAME_PLATE_UNIT_REMOVED(plateID) Visible_Plates[plateID] = nil end

    function LocalDatabase.NAME_PLATE_UNIT_ADDED(plateID)
        local unitPlate = C_NamePlate.GetNamePlateForUnit(plateID)

        Visible_Plates[plateID]             = {}
        Visible_Plates[plateID].unitFrame   = unitPlate.UnitFrame
        Visible_Plates[plateID].unitName    = UnitName(plateID) 
        Visible_Plates[plateID].unitGUID    = UnitGUID(plateID) 

        ScanForQuests(plateID) 
    end

--<<CONFIG RESET FUNCTION>>-------------------------------------------------------------------------<<>>

    function LocalDatabase.ResetAllQuestIcons()
        for _, plateFrame in pairs(Modified_Plates) do
            plateFrame.Quest_Icon:SetTexture("Interface\\AddOns\\Kib_QuestMobs\\media\\questicon_" .. SavedVars.TextureVariant)
            plateFrame.Quest_Icon:SetSize(DefaultWidth * SavedVars.ScaleOffset, DefaultHeight * SavedVars.ScaleOffset)

            if plateFrame.Quest_Icon:IsShown() then SetQuestIconColour(plateFrame) end
        end
    end

----------------------------------------------------------------------------------------------------<<END>>